#!/usr/bin/env python

import numpy as np
import scipy.io as sio
import sys
import os
caffe_root = '/your_path/caffe/'
sys.path.insert(0, caffe_root + 'python')
import caffe
caffe.set_mode_gpu()
#caffe.set_mode_cpu()

#set path for dataset and caffe prototxt/model
model_def = '/your_path/obj_centric.prototxt'
model_weights = '/your_path/object_centric.caffemodel'
image_dir = '/your_path/MIT_INDOOR/Images/'
train_list = '/your_path/MIT_indoor/TrainImages.txt'
test_list = '/your_path/MIT_indoor/TestImages.txt'
train_save = 'alexnet_train'
test_save = 'alexnet_test'

#generate labels for images
train_file = open(train_list,'r')
label_dic = {}
numOfClasses = 0
for line in train_file:
	line_parts = line.strip().split('/')
	if not label_dic.has_key(line_parts[0]):
		label_dic[line_parts[0]] = numOfClasses
		numOfClasses = numOfClasses + 1
train_file.close()

#initialize the network
net = caffe.Net(model_def,      # defines the structure of the model
                model_weights,  # contains the trained weights
                caffe.TEST)     # use test mode (e.g., don't perform dropout)

# load the mean ImageNet image (as distributed with Caffe) for subtraction
mu = np.load(caffe_root + 'python/caffe/imagenet/ilsvrc_2012_mean.npy')
mu = mu.mean(1).mean(1)  # average over pixels to obtain the mean (BGR) pixel values
print 'mean-subtracted values:', zip('BGR', mu)

# create transformer for the input called 'data'
transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
transformer.set_transpose('data', (2,0,1))  # move image channels to outermost dimension
transformer.set_mean('data', mu)            # subtract the dataset-mean value in each channel
transformer.set_raw_scale('data', 255)      # rescale from [0, 1] to [0, 255]
transformer.set_channel_swap('data', (2,1,0))  # swap channels from RGB to BGR

net.blobs['data'].reshape(1,        # batch size
                          3,         # 3-channel (BGR) images
                          227, 227)  # image size is 227x227

#get features and labels for training images
train_file = open(train_list,'r')
image_index = 0
num_of_trainImgs = 5360
train_pool5_features = np.empty([num_of_trainImgs,256*6*6])
train_labels = np.empty([num_of_trainImgs])
for line in train_file:
 	line = line.strip()
	filepath = image_dir + line
	line_parts = line.split('/')
	if filepath.endswith(".jpg"):
		image = caffe.io.load_image(filepath)
		transformed_image = transformer.preprocess('data', image)
		net.blobs['data'].data[...] = transformed_image
		output = net.forward()
		pool5_activations = net.blobs['pool5'].data
		pool5_feature = pool5_activations[0,:,:,:].flatten()
                #normalized_pool5 = (pool5_feature - pool5_feature.mean()) / pool5_feature.std()
                train_pool5_features[image_index,:] = pool5_feature
		train_labels[image_index] = label_dic[line_parts[0]]
		image_index = image_index + 1
train_labels = np.int_(train_labels)
np.savez(train_save, train_pool5_features = train_pool5_features, train_labels = train_labels)

#get features and labels for testing images
test_file = open(test_list,'r')
image_index = 0
num_of_testImgs = 1340
test_pool5_features = np.empty([num_of_testImgs,256*6*6])
test_labels = np.empty([num_of_testImgs])
for line in test_file:
	line = line.strip()
	filepath = image_dir + line
	line_parts = line.split('/')
	if filepath.endswith(".jpg"):
		image = caffe.io.load_image(filepath)
		transformed_image = transformer.preprocess('data', image)
		net.blobs['data'].data[...] = transformed_image
		output = net.forward()
		pool5_activations = net.blobs['pool5'].data
		pool5_feature = pool5_activations[0,:,:,:].flatten()
		#normalddized_pool5 = (pool5_feature - pool5_feature.mean()) / pool5_feature.std()
		test_pool5_features[image_index,:] = pool5_feature
		test_labels[image_index] = label_dic[line_parts[0]]
		image_index = image_index + 1
test_labels = np.int_(test_labels)
np.savez(test_save, test_pool5_features = test_pool5_features, test_labels = test_labels)
