#!/usr/bin/env python

import numpy as np
from sklearn import preprocessing

#load data
train_feat = np.load('features/train.npz')
train_pool5_features = train_feat['train_pool5_features']
y = train_feat['train_labels']
test_feat = np.load('features/test.npz')
test_pool5_features = test_feat['test_pool5_features']
y_test = test_feat['test_labels']

#preprocessing
std_scale = preprocessing.StandardScaler().fit(train_pool5_features)
train_pool5_features = std_scale.transform(train_pool5_features)
test_pool5_features = std_scale.transform(test_pool5_features)

#initialize the parameters
number_of_classes = 67
dim = 9216
W = 0.01 * np.random.randn(dim,number_of_classes)
b = np.zeros((1,number_of_classes))

#set some hyperparameters
step_size = 1
reg = 0.01
weight_decay = 0.1
niter = 2000

num_examples = train_pool5_features.shape[0]
for i in xrange(niter):
	#decay the step_size (learning rate)
	if i % 500 == 0:
		step_size = weight_decay * step_size

	#evaluate class scores
	scores = np.dot(train_pool5_features, W) + b

	#compute the class probabilities
	exp_scores = np.exp(scores)
	probs = exp_scores / np.sum(exp_scores, axis = 1, keepdims=True)

	#compute the loss: average cross-entropy loss and regularization
	correct_logprobs = -np.log(probs[range(num_examples),y])
	data_loss = np.sum(correct_logprobs)/num_examples
	reg_loss = 0.5*reg*np.sum(W*W)
	loss = data_loss + reg_loss
	if i % 10 == 0:
		print "iteration %d: loss %f" % (i,loss)
		scores = np.dot(train_pool5_features, W) + b
		predicted_class = np.argmax(scores, axis=1)
		print 'training accuracy: %.2f' % (np.mean(predicted_class == y))
		scores = np.dot(test_pool5_features, W) + b
		predicted_class = np.argmax(scores, axis = 1)
		print 'testing accuracy: %.2f' % (np.mean(predicted_class == y_test))

	#compute the gradient
	dscores = probs
	dscores[range(num_examples),y] -= 1
	dscores /= num_examples

	#backpropate the gradient to the parameters
	dW = np.dot(train_pool5_features.T, dscores)
	db = np.sum(dscores, axis=0, keepdims=True)
	dW += reg*W

	#perform a parameter updata
	W += -step_size * dW
	b += -step_size * db

#get accuracy on training data
scores = np.dot(train_pool5_features, W) + b
predicted_class = np.argmax(scores, axis=1)
print predicted_class
print y
print 'training accuracy: %.2f' % (np.mean(predicted_class == y))

#get accuracy on testing data
scores = np.dot(test_pool5_features, W) + b
predicted_class = np.argmax(scores, axis = 1)
print predicted_class
print y_test
print 'testing accuracy: %.2f' % (np.mean(predicted_class == y_test))
